###################################################
# Principles of Business Forecasting 2e, 2017
# ord, K., Fildes, R. and Kourentzes, N.
# (Book website) - TO BE EDITED
#
# Companion scripts for R
###################################################

# Exercises 10.1

###################################################
# These line will be removed when this is packaged!
load("./data/data_Compdata.Rdata")
###################################################

# Store the data in a conveniently named variable
X <- data_Compdata
n <- dim(X)[1]

# Randomly split sample in 80%-20%
idx <- sample(n,round(0.8*n))
X.trn <- X[idx,]              # training set
X.tst <- X[setdiff(1:n,idx),] # test set

# Question a
# Construct regression model
# Target is computeruser
fit <- lm(computeruser ~ inc1+inc2+inc3+inc4
                        +level2+level3+level4+prof,data=X.trn)
summary(fit)

# Evaluate estimation sample (training set) model output
X.trn.hat <- predict(fit)

# Apply cutoff and output classification matrix
cutoff <- c(0.5,0.2)
for (i in 1:length(cutoff)){
  # Translate model output into [0, 1]
  pred.trn <- X.trn.hat >= cutoff[i]
  # Produce matrix
  cmat <- rbind(c(sum(X.trn$computeruser==0 & pred.trn == 0),sum(X.trn$computeruser==0 & pred.trn == 1)),
                c(sum(X.trn$computeruser==1 & pred.trn == 0),sum(X.trn$computeruser==1 & pred.trn == 1)))
  # Add totals
  cmat <- cbind(cmat,rowSums(cmat))
  cmat <- rbind(cmat,colSums(cmat))
  colnames(cmat) <- c("Pred.0","Pred.1","Total")
  rownames(cmat) <- c("True.0","True.1","Total")
  print(paste("Cutoff:",cutoff[i]))
  print(cmat)
}

# Question b
# Produce a model only with income inputs
fit2 <- lm(computeruser ~ inc1+inc2+inc3+inc4,data=X.trn)
summary(fit2)

# Evaluate out-of-sample (test set) performance
X.tst.hat <- predict(fit,newdata=X.tst)
X2.tst.hat <- predict(fit2,newdata=X.tst)

# Results for full model
for (i in 1:length(cutoff)){
  # Translate model output into [0, 1]
  pred.tst <- X.tst.hat >= cutoff[i]
  # Produce matrix
  cmat <- rbind(c(sum(X.tst$computeruser==0 & pred.tst == 0),sum(X.tst$computeruser==0 & pred.tst == 1)),
                c(sum(X.tst$computeruser==1 & pred.tst == 0),sum(X.tst$computeruser==1 & pred.tst == 1)))
  # Add totals
  cmat <- cbind(cmat,rowSums(cmat))
  cmat <- rbind(cmat,colSums(cmat))
  colnames(cmat) <- c("Pred.0","Pred.1","Total")
  rownames(cmat) <- c("True.0","True.1","Total")
  print(paste("Cutoff:",cutoff[i]," - Full model"))
  print(cmat)
}

# Results for income only model
for (i in 1:length(cutoff)){
  # Translate model output into [0, 1]
  pred.tst <- X2.tst.hat >= cutoff[i]
  # Produce matrix
  cmat <- rbind(c(sum(X.tst$computeruser==0 & pred.tst == 0),sum(X.tst$computeruser==0 & pred.tst == 1)),
                c(sum(X.tst$computeruser==1 & pred.tst == 0),sum(X.tst$computeruser==1 & pred.tst == 1)))
  # Add totals
  cmat <- cbind(cmat,rowSums(cmat))
  cmat <- rbind(cmat,colSums(cmat))
  colnames(cmat) <- c("Pred.0","Pred.1","Total")
  rownames(cmat) <- c("True.0","True.1","Total")
  print(paste("Cutoff:",cutoff[i]," - Income only"))
  print(cmat)
}

# Question c
# Build a logistic regression model
# We now use the function glm
fit3 <- glm(computeruser ~ inc1+inc2+inc3+inc4+level2+level3+level4+prof,family=binomial(link='logit'),data=X.trn)
summary(fit3)
# Compare with initial linear regression model
summary(fit)
X3.tst.hat <- predict(fit3,newdata=X.tst,type='response')

# Results for logistic regression
for (i in 1:length(cutoff)){
  # Translate model output into [0, 1]
  pred.tst <- X3.tst.hat >= cutoff[i]
  # Produce matrix
  cmat <- rbind(c(sum(X.tst$computeruser==0 & pred.tst == 0),sum(X.tst$computeruser==0 & pred.tst == 1)),
                c(sum(X.tst$computeruser==1 & pred.tst == 0),sum(X.tst$computeruser==1 & pred.tst == 1)))
  # Add totals
  cmat <- cbind(cmat,rowSums(cmat))
  cmat <- rbind(cmat,colSums(cmat))
  colnames(cmat) <- c("Pred.0","Pred.1","Total")
  rownames(cmat) <- c("True.0","True.1","Total")
  print(paste("Cutoff:",cutoff[i]," - Logistic regression"))
  print(cmat)
}

# We have not considered pooling any regressors, for example as in section 10.3
# We can easily do that by combining for example income levels as follows:
X$incN <- X$inc1 + X$inc2
# This adds a new column to X with the sum of inc1 and inc2, which will be == 1 when either inc1 or inc2 were == 1
# Another way to do that is to use a logical or: X$inc1 | X$inc2
