###################################################
# Principles of Business Forecasting 2e, 2017
# ord, K., Fildes, R. and Kourentzes, N.
# (Book website) - TO BE EDITED
#
# Companion scripts for R
###################################################

# Exercises 10.2

###################################################
# These line will be removed when this is packaged!
load("./data/data_Compdata.Rdata")
###################################################

# Store the data in a conveniently named variable
X <- data_Compdata
n <- dim(X)[1]

# Randomly split sample in 80%-20%
idx <- sample(n,round(0.8*n))
X.trn <- X[idx,]              # training set
X.tst <- X[setdiff(1:n,idx),] # test set

# To build decision trees we will use function rpart from the rpart package
if (!require("rpart")){install.packages("rpart");library(rpart)}

fit <- rpart(computeruser ~ inc1+inc2+inc3+inc4+level2+level3+level4+prof, data=X.trn, method="class")
# Printing the fitted tree gives a quick summary of the decision rules
print(fit)
# Asking for a summary provides a more detailed output of the fitted tree
summary(fit)
# A graphical summmary can be obtained by plot and text
plot(fit)
text(fit)

# To predict the classification we still use the function predict
X.tst.pred <- predict(fit,newdata=X.tst)
# Observe that this provides two columns, one for the probability of the result being class 1 and one for the probability of being class 2.
print(X.tst.pred)
# We will retain only the second column, and from that point on the result can be used in the same way as the output of lm or glm.
X.tst.pred <- X.tst.pred[,2]

# Results for decision tree
for (i in 1:length(cutoff)){
  # Translate model output into [0, 1]
  pred.tst <- X.tst.hat >= cutoff[i]
  # Produce matrix
  cmat <- rbind(c(sum(X.tst$computeruser==0 & pred.tst == 0),sum(X.tst$computeruser==0 & pred.tst == 1)),
                c(sum(X.tst$computeruser==1 & pred.tst == 0),sum(X.tst$computeruser==1 & pred.tst == 1)))
  # Add totals
  cmat <- cbind(cmat,rowSums(cmat))
  cmat <- rbind(cmat,colSums(cmat))
  colnames(cmat) <- c("Pred.0","Pred.1","Total")
  rownames(cmat) <- c("True.0","True.1","Total")
  print(paste("Cutoff:",cutoff[i]," - Full model"))
  print(cmat)
}

# Let us evaluate the effect of combining the two lowest classes of income
X.trn$incN <- as.numeric(X.trn$inc1 | X.trn$inc2)
X.tst$incN <- as.numeric(X.tst$inc1 | X.tst$inc2)
fit2 <- rpart(computeruser ~ incN+inc3+inc4+level2+level3+level4+prof, data=X.trn, method="class")
summary(fit2)
X2.tst.pred <- predict(fit2,newdata=X.tst)[,2]

for (i in 1:length(cutoff)){
  # Translate model output into [0, 1]
  pred.tst <- X2.tst.hat >= cutoff[i]
  # Produce matrix
  cmat <- rbind(c(sum(X.tst$computeruser==0 & pred.tst == 0),sum(X.tst$computeruser==0 & pred.tst == 1)),
                c(sum(X.tst$computeruser==1 & pred.tst == 0),sum(X.tst$computeruser==1 & pred.tst == 1)))
  # Add totals
  cmat <- cbind(cmat,rowSums(cmat))
  cmat <- rbind(cmat,colSums(cmat))
  colnames(cmat) <- c("Pred.0","Pred.1","Total")
  rownames(cmat) <- c("True.0","True.1","Total")
  print(paste("Cutoff:",cutoff[i]," - Full model"))
  print(cmat)
}

# As a final note, a step beyond decision trees are random forests, which can be easily implemented with the
# randomForest package
