###################################################
# Principles of Business Forecasting 2e, 2017
# ord, K., Fildes, R. and Kourentzes, N.
# (Book website) - TO BE EDITED
#
# Companion scripts for R
###################################################

# Exercises 10.3

###################################################
# These line will be removed when this is packaged!
load("./data/data_Compdata.Rdata")
###################################################

# Store the data in a conveniently named variable
X <- data_Compdata
n <- dim(X)[1]

# Randomly split sample in 80%-20%
idx <- sample(n,round(0.8*n))
X.trn <- X[idx,]              # training set
X.tst <- X[setdiff(1:n,idx),] # test set

# To build neural networks we will use function neuralnet from the neuralnet package
if (!require("neuralnet")){install.packages("neuralnet");library(neuralnet)}

# We control the number of hidden nodes with the argument hidden
# Also since we are dealing with a classification problem, it makes sense to use a sigmoid output layer.
# We can do that by setting linear.output=FALSE
# We will fit all five networks (different initialisations) in one go
fit <- pred.trn <- pred.tst <- vector("list",5)
for (i in 1:5){
  # Fit the network
  fit[[i]] <- neuralnet(computeruser~inc1+inc2+inc3+inc4+level2+level3+level4+prof, data=X.trn, hidden=3, linear.output=FALSE)
  # To produce the model fit we use the function compute
  # However, in contrast to predict, we need to indicate which variables go in the model
  pred.trn[[i]] <- compute(fit[[i]],cbind(X.trn$inc1,X.trn$inc2,X.trn$inc3,X.trn$inc4,X.trn$level2,X.trn$level3,X.trn$level4,X.trn$prof))$net.result
  pred.tst[[i]] <- compute(fit[[i]],cbind(X.tst$inc1,X.tst$inc2,X.tst$inc3,X.tst$inc4,X.tst$level2,X.tst$level3,X.tst$level4,X.tst$prof))$net.result
}
# The network and the weights can be visualised with plot
plot(fit[[1]])

# Now let us evaluate the alternative classification options
# First option is take the average of the predicted values and set it to 1 if the result is above 0.8
pred.tst.A <- matrix(unlist(pred.tst), ncol = 5)
pred.tst.A <- apply(pred.tst.A,1,mean)
pred.tst.A <- pred.tst.A > 0.8

# Produce classification matrix
pred <- pred.tst.A
cmat.A <- rbind(c(sum(X.tst$computeruser==0 & pred == 0),sum(X.tst$computeruser==0 & pred == 1)),
                c(sum(X.tst$computeruser==1 & pred == 0),sum(X.tst$computeruser==1 & pred == 1)))
# Add totals
cmat.A <- cbind(cmat.A,rowSums(cmat.A))
cmat.A <- rbind(cmat.A,colSums(cmat.A))
colnames(cmat.A) <- c("Pred.0","Pred.1","Total")
rownames(cmat.A) <- c("True.0","True.1","Total")
print(cmat.A)

# Another is to use each network on its own
cmat <- vector("list",5)
for (i in 1:5){
  cmat[[i]] <- rbind(c(sum(X.tst$computeruser==0 & (pred.tst[[i]]>0.8) == 0),sum(X.tst$computeruser==0 & (pred.tst[[i]]>0.8) == 1)),
                     c(sum(X.tst$computeruser==1 & (pred.tst[[i]]>0.8) == 0),sum(X.tst$computeruser==1 & (pred.tst[[i]]>0.8) == 1)))
  # Add totals
  cmat[[i]] <- cbind(cmat[[i]],rowSums(cmat[[i]]))
  cmat[[i]] <- rbind(cmat[[i]],colSums(cmat[[i]]))
  colnames(cmat[[i]]) <- c("Pred.0","Pred.1","Total")
  rownames(cmat[[i]]) <- c("True.0","True.1","Total")
  print(cmat[[i]])
}

# At this point we need to decide which of the five training initialisations is better.
# There are many ways to do that.
# Since we are using networks with the same specification we could use in-sample MSE to rank them
MSE <- colMeans((matrix(rep(X.trn$computeruser,5),ncol=5) - matrix(unlist(pred.trn), ncol = 5))^2)
idx <- which(MSE==min(MSE))[1]
paste("Lowest MSE is achieved by network",idx)
print(cmat[[idx]])

# Another way is to plot the ROC
# We can calculate ROC using the function roc from the pROC package
if (!require("pROC")){install.packages("pROC");library(pROC)}
if (!require("RColorBrewer")){install.packages("RColorBrewer");library(RColorBrewer)}
ROC <- vector("list",5)
cmp <- brewer.pal(5,"Set1")
for (i in 1:5){
  ROC[[i]] <- roc(as.vector(X.tst$computeruser),as.vector(pred.tst[[i]]))
  if (i == 1){
    plot(ROC[[i]],col=cmp[[i]])
  } else {
    plot(ROC[[i]],add=TRUE,col=cmp[[i]])
  }
}
legend("topleft",paste0("Init.",1:5),lty=1,col=cmp)

# Or we can calculate the Area Under ROC (AUROC) with function auc from the pROC package
AUC <- vector("numeric",5)
for (i in 1:5){
  AUC[i] <- auc(ROC[[i]])
}
idx2 <- which(AUC==max(AUC))[1]
paste("Maximum AUC is achieved by network",idx2)
print(cmat[[idx2]])

# We can now compare the best network result, with the average performance and the ensemble prediction (average of responses)
