###################################################
# Principles of Business Forecasting 2e, 2017
# ord, K., Fildes, R. and Kourentzes, N.
# (Book website) - TO BE EDITED
#
# Companion scripts for R
###################################################

# Exercises 2.11

###################################################
# These line will be removed when this is packaged!
load("./data/data_Boulder_2.Rdata")
###################################################

# Store the data in a conveniently named variable
y <- data_Boulder_2

# Produce forecasts copying the previous year (naive seasonal forecast)
n <- length(y)
# first year is NAs as we do not know the past and the rest is just a copy of
# the historical values, ignoring the last 12 months.
frc1 <- c(rep(NA,12),y[-((n-11):n)])

# Produce the forecast using the monthly averages
# Recall from exercise 2.4
z <- matrix(y,nrow=12)
rownames(z) <- month.abb
colnames(z) <- unique(format(as.Date(time(y), format="%d/%m/%Y"),"%Y"))
mnth_average <- apply(z,1,mean)
# Repeat accordingly to produce forecasts for all years
frc2 <- c(rep(NA,12), rep(mnth_average,(n/12)-1))

# Let us plot the forecasts and actuals to get an initial feel of accuracy
frc1 <- ts(frc1,frequency=frequency(y),start=start(y))
frc2 <- ts(frc2,frequency=frequency(y),start=start(y))
# For colouring I will use the excellent RColorBrewer package
# If you prefer not to use it, simply run: cmp <- c("black","blue","red")
if (!require("RColorBrewer")){install.packages("RColorBrewer")}
cmp <- brewer.pal(3,"Set1")
ts.plot(cbind(y,frc1,frc2),col=cmp)
legend("topleft",c("Actual","S.Naive","Averages"),col=cmp,lty=1)

# Calculate errors
e <- cbind(y-frc1,y-frc2)
colnames(e) <- c("S.Naive","Averages")

# Let us calculate the denominators for RelMAE and RelMdAE (naive forecast)
q <- abs(y[13:n] - y[12:(n-1)])
naive_e <- c(mean(q),median(q))

E <- rbind(
           # Calculate ME and MdE
           apply(e,2,mean,na.rm=TRUE),
           apply(e,2,median,na.rm=TRUE),
           # Calculate MAE and MdAE
           apply(abs(e),2,mean,na.rm=TRUE),
           apply(abs(e),2,median,na.rm=TRUE),
           # Calculate RMSE and RMdSE
           sqrt(apply(e^2,2,mean,na.rm=TRUE)),
           sqrt(apply(e^2,2,median,na.rm=TRUE)),
           # Caclulate RelMAE and RelMdAE
           apply(abs(e),2,mean,na.rm=TRUE)/naive_e[1],
           apply(abs(e),2,median,na.rm=TRUE)/naive_e[2])
# Name the errors (rows)
rownames(E) <- c("ME","MdE","MAE","MdAE","RMSE","RMdSE","RelMAE","RelMdAE")
# Print results
print(round(E,3))
