###################################################
# Principles of Business Forecasting 2e, 2017
# ord, K., Fildes, R. and Kourentzes, N.
# (Book website) - TO BE EDITED
#
# Companion scripts for R
###################################################

# Exercises 2.14

###################################################
# These line will be removed when this is packaged!
load("./data/data_Boulder_2.Rdata")
###################################################

# Store the data in a conveniently named variable
y <- data_Boulder_2

# We illustrate the solution for the seasonal naive forecast of 2.11
n <- length(y)
# As before, first year is NAs as we do not know the past and the rest is just a copy of
# the historical values, ignoring the last 12 months. Now we keep an additional 12 months as holdout to do the comparison
insample_frc <- c(rep(NA,12),y[-((n-11-12):(n-12))])
outofsample_frc <- y[(n-11-12):(n-12)]

# Construct prediction intervals
a <- 0.95
e <- y - insample_frc
e <- e[!is.na(e)]
# Normal
RMSE <- sqrt(mean(e^2))
PInormU <- outofsample_frc + abs(qnorm((1-a)/2))*RMSE
PInormL <- outofsample_frc - abs(qnorm((1-a)/2))*RMSE
# Empirical
PIempU <- outofsample_frc + quantile(e,a+(1-a)/2)
PIempL <- outofsample_frc + quantile(e,(1-a)/2)

# Compare
actuals <- window(y,start=c(2015,1),end=c(2015,12))
# Normal
PInormU < actuals
PInormL > actuals
# Empirical
PIempU < actuals
PIempL > actuals

# Visualise
if (!require("RColorBrewer")){install.packages("RColorBrewer")}
cmp <- brewer.pal(3,"Set1")
yy <- range(c(y,PIempU,PInormU,PIempL,PInormL))
ts.plot(window(y,start=c(2013,1),end=c(2014,12)),window(y,start=c(2015,1),end=c(2015,12)),col=c("gray60","black"),ylim=yy)
xx <- as.vector(time(window(y,start=c(2015,1),end=c(2015,12))))
lines(xx,PIempU,col=cmp[1],lty=2)
lines(xx,PIempL,col=cmp[1],lty=2)
lines(xx,PInormU,col=cmp[2],lty=2)
lines(xx,PInormL,col=cmp[2],lty=2)
lines(xx,outofsample_frc,col=cmp[3])
legend("topleft",c("Historical","Holdout","PI Empirical","PI Normal","Forecast"),col=c("gray60","black",cmp),lty=1)
