###################################################
# Principles of Business Forecasting 2e, 2017
# ord, K., Fildes, R. and Kourentzes, N.
# (Book website) - TO BE EDITED
#
# Companion scripts for R
###################################################

# Exercises 3.10

###################################################
# These line will be removed when this is packaged!
load("./data/data_Netflix_2.Rdata")
###################################################

if (!require("forecast")){install.packages("forecast")}

# Store the data in a conveniently named variable
y <- data_Netflix_2

# Let us use estimation samples starting from 2000 up to 2009, ending at 2010 and test the accuracy on 2011-2015.
# of course we can set any ranges we want, but note that the test set must be the same for the results to be
# comparable.
strt <- 2000:2009
l <- length(strt)

# For simplicity we will also consider only the Total Revenues, but we can easily extend the solution to all 3 series in the dataset
# We will store MAE for each sample size
MAE <- array(NA,c(l,2),dimnames=list(strt,c("SES","LES")))
param <- array(NA,c(l,3),dimnames=list(strt,c("SES-a","LES-a","LES-b")))

# We will automate the calculation
for (q in 1:l){
  # Extract sample to work with
  z <- window(y[,1],start=strt[q])
  n <- length(z)

  # Let us find how long is the evaluation set
  test <- window(z,start=2011) # Set the start date of the test set, this will also limit the fitting sample
  m <- length(test)

  # Fit optimal SES and LES
  k <- 2
  fit <- vector("list",k)
  fit[[1]] <- ets(window(z,end=c(2010,4)),model="ANN")
  fit[[2]] <- ets(window(z,end=c(2010,4)),model="AAN")

  # NOw perform a rolling origin evaluation with these two model (We will not re-optimse, so that
  # we make tracking of the parameters easy - we could easily extend this).

  frc <- array(NA,c(m,k),dimnames=list(time(test),c("SES","LES")))
  for (i in 1:m){
    fitsample <- z[1:(n-m+i-1)]
    fitsample <- ts(fitsample,frequency=frequency(y),start=start(y))
    for (j in 1:k){
      fitrol <- ets(fitsample,model=fit[[j]],use.initial.values=TRUE)
      frc[i,j] <- forecast(fitrol,h=1)$mean
    }
  }

  # Calculate errors
  e <- matrix(rep(test,k),ncol=k) - frc
  MAE[q,] <- apply(abs(e),2,mean)
  # Store parameters
  param[q,1] <- fit[[1]]$par[1]
  param[q,2:3] <- fit[[2]]$par[1:2]

}

# Provide some descriptives on errors and parameters
summary(MAE)
summary(param)

# Visualise parameters across estimation samples
boxplot(param)
