###################################################
# Principles of Business Forecasting 2e, 2017
# ord, K., Fildes, R. and Kourentzes, N.
# (Book website) - TO BE EDITED
#
# Companion scripts for R
###################################################

# Exercises 3.16

###################################################
# These line will be removed when this is packaged!
load("./data/data_Dulles_2.Rdata")
###################################################

if (!require("forecast")){install.packages("forecast")}
if (!require("RColorBrewer")){install.packages("RColorBrewer")}
cmp <- brewer.pal(5,"Set1")

# Store the data in a conveniently named variable
y <- data_Dulles_2
a <- 0.95

# For this exercise we merely need to restrict the starting date of the time series
# and the rest is the same as for exercise 2.11
y <- window(y,start=1986)
n <- length(y)

# The different estimation samples can be solved as in 3.10
# Here we will quickly use the two separate data ranges

teststr <- c(2004,2011)
l <- length(teststr)

RMSE <- MAE <- RelMAE <- MAPE <- array(NA,c(l,3),dimnames=list(c("2004-2015","2011-2015"),c("LES","DES","LES-log")))

# We will automate the calculation
for (i in 1:l){

  # Specify the test set
  test <- window(y,start=teststr[i])
  m <- length(test)

  # Fit models
  k <- 3
  fit <- vector("list",k)
  # Linear trend exponential smoothing
  fit[[1]] <- ets(window(y,end=teststr[i]-1),model="AAN",damped=FALSE)
  # Damped trend exponential smoothing
  fit[[2]] <- ets(window(y,end=teststr[i]-1),model="AAN",damped=TRUE)
  # Linear trend exponential smoothing on log-transformed data
  fit[[3]] <- ets(log(window(y,end=teststr[i]-1)),model="AAN",damped=FALSE)
  # Get in-sample RMSE of models for calculation of PIs
  S <- unlist(lapply(fit,function(x){sqrt(x$mse)}))

  # Produce forecasts
  frc <- PIlower <- PIupper <- array(NA,c(m,k),dimnames=list(time(test),colnames(RMSE)))
  for (j in 1:m){
    # We run a rolling origin evaluation. At each iteration (j) we need to provide the current fitting sample
    # This must be a ts class object for the forecast package functions to work properly
    fitsample <- y[1:(n-m+j-1)]
    fitsample <- ts(fitsample,frequency=frequency(y),start=start(y))
    for (q in 1:k){
      # First we fit ets with the fixed alpha value and model="AAN" - we will let it optimise the initial trend
      if (q <= 2){
        fitrol <- ets(fitsample,model=fit[[q]],use.initial.values=TRUE)
        frc[j,q] <- forecast(fitrol,h=1)$mean
        PIlower[j,q] <- frc[j,q] - abs(qnorm((1-a)/2))*S[q]
        PIupper[j,q] <- frc[j,q] + abs(qnorm((1-a)/2))*S[q]
      } else {
        fitrol <- ets(log(fitsample),model=fit[[q]],use.initial.values=TRUE)
        frc[j,q] <- exp(forecast(fitrol,h=1)$mean)
        PIlower[j,q] <- exp(log(frc[j,q]) - abs(qnorm((1-a)/2))*S[q])
        PIupper[j,q] <- exp(log(frc[j,q]) + abs(qnorm((1-a)/2))*S[q])
      }
    }
  }

  frc <- ts(frc,frequency=frequency(test),start=start(test))
  PIlower <- ts(PIlower,frequency=frequency(test),start=start(test))
  PIupper <- ts(PIupper,frequency=frequency(test),start=start(test))
  e <- matrix(rep(test,k),ncol=k) - frc
  RMSE[i,] <- sqrt(apply(e^2,2,mean))
  MAE[i,] <- apply(abs(e),2,mean)
  RelMAE[i,] <- MAE[i,]/mean(abs(y[(n-m+1):n] - y[(n-m):(n-1)]))
  MAPE[i,] <- 100*apply(abs(e)/matrix(rep(test,k),ncol=k),2,mean)

  # Visualise PI calclulations
  yy <- range(c(test,PIupper,PIlower))
  plot(test,ylim=yy,ylab="Passengers (000s)",main=paste("Test set ",min(time(test)),"-",max(time(test))))
  for (q in 1:k){
    lines(frc[,q],col=cmp[q])
    lines(PIupper[,q],col=cmp[q],lty=2)
    lines(PIlower[,q],col=cmp[q],lty=2)
  }
  legend("topleft",c("Out-of-sample",colnames(RMSE)),col=c("black",cmp[1:3]),lty=1)


}

print(round(RMSE,3))
print(round(MAE,3))
print(round(RelMAE,3))
print(round(MAPE,3))
