###################################################
# Principles of Business Forecasting 2e, 2017
# ord, K., Fildes, R. and Kourentzes, N.
# (Book website) - TO BE EDITED
#
# Companion scripts for R
###################################################

# Exercises 3.3

###################################################
# These line will be removed when this is packaged!
load("./data/data_GDP_change_2.Rdata")
###################################################

# Store the data in a conveniently named variable
y <- data_GDP_change_2
y <- window(y,start=start(y),end=2015)
n <- length(y)

# Let us find how long is the evaluation set
test <- window(y,start=2001)
m <- length(test)

# Initialise an array to store the forecasts
alpha <- c(0.2,0.5,0.8)
k <- length(alpha)
frc <- array(NA,c(m,k),dimnames=list(time(test),paste0("SES (a=",alpha,")")))

# We will rely on the ets function in the forecast package (ETS: state-space exponential smoothing model).
# An alternative option is the es function in the smooth package
# Just for SES there is a third option in the tsintermittent package, with the function sexsm
if (!require("forecast")){install.packages("forecast")}

# Neither ets not es support manual starting values, and instead use various approaches to get them.
# Instead of programming single exponential smoothing from scratch, we will use a simple trick to
# force whatever initial values we want on ets - given fixed alpha.

# Produce forecasts
for (i in 1:m){
  # We run a rolling origin evaluation. At each iteration (j) we need to provide the current fitting sample
  # This must be a ts class object for the forecast package functions to work properly
  fitsample <- y[1:(n-m+i-1)]
  fitsample <- ts(fitsample,frequency=frequency(y),start=start(y))
  for (j in 1:k){
    # First we fit ets with the fixed alpha value and model="ANN" - this is SES.
    fit <- ets(fitsample,model="ANN",alpha=alpha[j])
    # Now we change the initial value in the fitted model...
    fit$initstate[1] <- window(y,start=1963,end=1963)
    # ... and recalculate
    fit <- ets(fitsample,model=fit,use.initial.values=TRUE)
    # This trick can work with any exponential smoothing model

    # Produce the 1-step ahead forecast
    frc[i,j] <- forecast(fit,h=1)$mean
  }
}

# To simplify plotting the results we convert frc into a ts object
frc <- ts(frc,frequency=frequency(test),start=start(test))

# Produce plot
if (!require("RColorBrewer")){install.packages("RColorBrewer")}
cmp <- brewer.pal(3,"Set1")

plot(y,ylab="GDP change")
for (j in 1:k){
  lines(frc[,j],col=cmp[j])
}
legend("topright",c("Data",colnames(frc)),col=c("black",cmp),lty=1)

# Calculate error metrics
e <- matrix(rep(test,k),ncol=k) - frc
# RMSE & MAE
RMSE <- sqrt(apply(e^2,2,mean))
MAE <- apply(abs(e),2,mean)
E <- rbind(RMSE,MAE)
print(round(E,3))

