###################################################
# Principles of Business Forecasting 2e, 2017
# ord, K., Fildes, R. and Kourentzes, N.
# (Book website) - TO BE EDITED
#
# Companion scripts for R
###################################################

# Exercises 3.4

###################################################
# These line will be removed when this is packaged!
load("./data/data_GDP_change_2.Rdata")
###################################################

# Store the data in a conveniently named variable
y <- data_GDP_change_2
y <- window(y,start=start(y),end=2015)
n <- length(y)

# Let us find how long is the evaluation set
test <- window(y,start=2001)
m <- length(test)

# Initialise an array to store the forecasts
frc <- array(NA,c(m,1),dimnames=list(time(test),"SES"))

# With the ets function we cannot calculate optimal alpha and fix the initial level.
# Instead we will let it optimise all initial values and smoothing parameters.
# The sexsm function from the tsintermittent package permits this, but for the sake
# of consistency we will stick to the ets function from the forecast package.
if (!require("forecast")){install.packages("forecast")}

# Produce forecasts
for (i in 1:m){
  # We run a rolling origin evaluation. At each iteration (j) we need to provide the current fitting sample
  # This must be a ts class object for the forecast package functions to work properly
  fitsample <- y[1:(n-m+i-1)]
  fitsample <- ts(fitsample,frequency=frequency(y),start=start(y))
  # Fit ets with model="ANN"
  fit <- ets(fitsample,model="ANN")
  # Produce the 1-step ahead forecast
  frc[i,1] <- forecast(fit,h=1)$mean
}

# To simplify plotting the results we convert frc into a ts object
frc <- ts(frc,frequency=frequency(test),start=start(test))

# Produce plot
if (!require("RColorBrewer")){install.packages("RColorBrewer")}
cmp <- brewer.pal(3,"Set1")

plot(y,ylab="GDP change")
lines(frc[,1],col=cmp[1])
legend("topright",c("Data",colnames(frc)),col=c("black",cmp),lty=1)

# Calculate error metrics
e <- test - frc
# RMSE & MAE
RMSE <- sqrt(apply(e^2,2,mean))
MAE <- apply(abs(e),2,mean)
E <- rbind(RMSE,MAE)
print(round(E,3))

