###################################################
# Principles of Business Forecasting 2e, 2017
# ord, K., Fildes, R. and Kourentzes, N.
# (Book website) - TO BE EDITED
#
# Companion scripts for R
###################################################

# Exercises 4.1

###################################################
# These line will be removed when this is packaged!
load("./data/data_Walmart_2.Rdata")
###################################################

# Store the data in a conveniently named variable
y <- data_Walmart_2[,1]

# Figures 4.2 and 4.3 provide helpful guidance for exploring the time series
# First let us produce all plots manually and then examine available functions

# We convert the time series to a matrix of values, where each column is a year
z <- matrix(y,nrow=4)
rownames(z) <- paste0("Q",1:4)
colnames(z) <- unique(trunc(time(y)))
k <- dim(z)[2] # Number of years

# Get some colours for the plots
if (!require("RColorBrewer")){install.packages("RColorBrewer")}
cmp <- colorRampPalette(brewer.pal(9,"Blues")[5:8])(k)

# Produce the first seasonal plot
yy <- range(y)
plot(NA,NA,xlim=c(1,4),ylim=yy,xlab="Quarters",ylab="Sales")
for (i in 1:k){
  lines(z[,i],col=cmp[i],type="o",pch=20)
}
legend("topleft",legend=range(unique(trunc(time(y)))),col=cmp[c(1,k)],lty=1,bty="n")

# Produce second plot
xt <- time(y)
xx <- range(xt)
plot(NA,NA,xlim=xx,ylim=yy,xlab="Period",ylab="Sales")
for (i in 1:k){
  lines(xt[(1+(i-1)*4):(4+(i-1)*4)],z[,i],col=cmp[i])
  if (i < k){
    lines(xt[(4+(i-1)*4):(5+(i-1)*4)],y[(4+(i-1)*4):(5+(i-1)*4)],col=cmp[i],lty=2)
  }
}

###################################################
# Optional steps
###################################################

# The function seasonplot from the forecast package is useful to construct seasonal plots
if (!require("forecast")){install.packages("forecast")}
seasonplot(y)

# An alternative function to produce seasonal plots can be found in the package TStools
if (!require("TStools")){
  if (!require("devtools")){install.packages("devtools")}
  devtools::install_github("trnnick/TStools")}

# Now we call function seasplot
seasplot(y)
# Note that this function by default removes any trend in the time series. You can switch that behaviour off. See ?seasplot
# With seasplot we can produce several variants of the seasonal plot easily
# Seasonal boxplots
seasplot(y,outplot=2)
# Seasonal subseries plot
seasplot(y,outplot=3)
# Seasonal distribution plot
seasplot(y,outplot=4)
