###################################################
# Principles of Business Forecasting 2e, 2017
# ord, K., Fildes, R. and Kourentzes, N.
# (Book website) - TO BE EDITED
#
# Companion scripts for R
###################################################

# Exercises 4.9

# There is no default function that implements pure seasonal exponential smoothing, so we will have to construct this manually.
# First, let us get the data
y <- c(115,90,65,135,130,95,75,150,135,105,85,155,145,110,85,160)
y <- ts(y,frequency=4)
plot(y)

# Produce forecasts using multiplicative decomposition
# We will perform the decomposition manually

# First we need to calculate the centred moving average (CMA)
# For this we can use the ma function from the forecast package
if (!require("forecast")){install.packages("forecast")}

# Calculate CMA
cma <- ma(y,4)
# We now divide the series to get the detrended one
y_detrend <- y/cma
# We calculate averages per quarter
season <- apply(matrix(y_detrend,ncol=4,byrow=TRUE),2,mean,na.rm=TRUE)
# Adjust the seasonal indices
season <- season/mean(season)

# The series is now decomposed. To forecast the series, first we will deseasonalise it, forecast the underlying trend and reseasonalise

# Deseasonalise
y_sa <- y/rep(season,length(y)/4)
plot(y_sa)
# Forecast trend
fit <- ets(y_sa,model="AZN")
frc <- forecast(fit,h=4)$mean
# Re-seasonalise
frc <- frc * season

# Plot result
ts.plot(y,frc,col=c("black","red"))
