###################################################
# Principles of Business Forecasting 2e, 2017
# ord, K., Fildes, R. and Kourentzes, N.
# (Book website) - TO BE EDITED
#
# Companion scripts for R
###################################################

# Exercises 5.10
# Exercises 5.11-5.14 follow a similar logic

###################################################
# These line will be removed when this is packaged!
load("./data/data_Job_openings.Rdata")
###################################################

if (!require("forecast")){install.packages("forecast")}

# Store the data in a conveniently named variable
y <- data_Job_openings
n <- length(y)

# Model repertoire to evaluate
models <- c("AAN","MAN","MMN","AAA","MAA","MAM","MMM")
# Damped trend is controlled by the argument damped
# We will run all models with and without damped trend
damped <- c(rep(FALSE,rep(length(models))),rep(TRUE,rep(length(models))))
models <- rep(models,2)
models <- c("ANN","MNN",models)
damped <- c(FALSE,FALSE,damped)
k <- length(models)

# We will consider the Akaike's Information Criterion.
# When we use this we do not need to any validation/hold-out set
# so we will use the complete available sample
AIC <- array(NA,c(k,1),dimnames=list(models,"AIC"))
for (i in 1:k){
  fit <- ets(y,model=models[i],damped=damped[i])
  AIC[i] <- fit$aic
}

loc_best_AIC <- which(AIC == min(AIC))
paste0("The best model according to AIC is ",models[loc_best_AIC],if(damped[loc_best_MAE]==TRUE){" with damped trend"},".")

# For the hold-out we will perform a rolling origin evaluation
# And to make our comparison more flexible we will allow the user to set
# the forecast horizon
# Here we will set it to 1 quarter
horizon <- 1

# Let us find how long is the evaluation set
test <- window(y,start=tail(time(y),36)[1])
m <- length(test)

# Produce forecasts and calculate holdout accuracy (MAE)
mae <- array(NA,c(m-horizon+1,k))
for (i in 1:(m-horizon+1)){
  fitsample <- y[1:(n-m+i-1)]
  fitsample <- ts(fitsample,frequency=frequency(y),start=start(y))
  for (j in 1:k){
    fit <- ets(fitsample,model=models[j],damped=damped[j])
    frc <- forecast(fit,h=horizon)$mean
    mae[i,j] <- mean(abs(y[(n-m+i):(n-m+i+horizon-1)] - frc))
  }
}
MAE <- apply(mae,2,mean)

loc_best_MAE <- which(MAE == min(MAE))
paste0("The best model according to holdout MAE is ",models[loc_best_MAE],if(damped[loc_best_MAE]==TRUE){" with damped trend"},".")

# Now we calculate model fit on the same sample
# and construct prediction intervals for 1-step ahead forecasts
a <- .95

frc <- upper <- lower <- array(NA,c(m,2),dimnames=list(time(test),c("AIC","MAE")))
for (i in 1:m){
  fitsample <- y[1:(n-m+i-1)]
  fitsample <- ts(fitsample,frequency=frequency(y),start=start(y))
  # Refit models
  fit_aic <- ets(fitsample,model=models[loc_best_AIC],damped=damped[loc_best_AIC])
  fit_mae <- ets(fitsample,model=models[loc_best_MAE],damped=damped[loc_best_MAE])
  # Produce forecasts
  frc_aic <- forecast(fit_aic,h=1,level=a*100)
  frc_mae <- forecast(fit_mae,h=1,level=a*100)
  frc[i,] <- c(frc_aic$mean,frc_mae$mean)
  upper[i,] <- c(frc_aic$upper,frc_mae$upper)
  lower[i,] <- c(frc_aic$lower,frc_mae$lower)
}

# Visualise results
if (!require("RColorBrewer")){install.packages("RColorBrewer")}
cmp <- brewer.pal(3,"Set1")
frc <- ts(frc,frequency=frequency(test),start=start(test))
upper <- ts(upper,frequency=frequency(test),start=start(test))
lower <- ts(lower,frequency=frequency(test),start=start(test))
yy <- range(c(y,upper,lower))
plot(y,ylim=yy)
for (i in 1:2){
  lines(frc[,i],col=cmp[i])
  lines(lower[,i],col=cmp[i],lty=2)
  lines(upper[,i],col=cmp[i],lty=2)
}
legend("topleft",c("AIC","MAE"),col=cmp[1:k],lty=1)

# Calculate forecast accuracy
testMAE <- apply(abs(matrix(rep(test,2),ncol=2) - frc),2,mean)
print(testMAE)

