###################################################
# Principles of Business Forecasting 2e, 2017
# ord, K., Fildes, R. and Kourentzes, N.
# (Book website) - TO BE EDITED
#
# Companion scripts for R
###################################################

# Exercises 5.8

###################################################
# These line will be removed when this is packaged!
load("./data/data_Vix.Rdata")
###################################################

if (!require("forecast")){install.packages("forecast")}

# Store the data in a conveniently named variable
y <- data_Vix[,4]
n <- length(y)
# Apply the log transformation
y <- log(y)

# Let us find how long is the evaluation set
test <- window(y,start=c(2013,1))
m <- length(test)

# Initialise an array to store the forecasts
models <- c("ANN","AAN","MNN","MMN")
k <- length(models)
frc <- lower <- upper <- array(NA,c(m,k),dimnames=list(time(test),models))

# Produce forecasts
for (i in 1:m){
  fitsample <- y[1:(n-m+i-1)]
  fitsample <- ts(fitsample,frequency=frequency(y),start=start(y))
  for (j in 1:k){
    # Fit model - Note: we reoptimise the ETS model as we roll the forecast origin
    fit <- ets(fitsample,model=models[j])
    # Produce the 1-step ahead forecast
    temp <- forecast(fit,h=1,level=95)
    frc[i,j] <- temp$mean
    lower[i,j] <- temp$lower
    upper[i,j] <- temp$upper
  }
}

# Visualise results
if (!require("RColorBrewer")){install.packages("RColorBrewer")}
cmp <- brewer.pal(max(3,k),"Set1")
frc <- ts(frc,frequency=frequency(test),start=start(test))
upper <- ts(upper,frequency=frequency(test),start=start(test))
lower <- ts(lower,frequency=frequency(test),start=start(test))
yy <- range(c(y,upper,lower))
plot(y,ylim=yy)
for (i in 1:k){
  lines(frc[,i],col=cmp[i])
  lines(lower[,i],col=cmp[i],lty=2)
  lines(upper[,i],col=cmp[i],lty=2)
}
legend("topleft",models,col=cmp[1:k],lty=1)

# Check how many observations fall outside the PIs
outsidePI <- array(NA,c(k,1),dimnames=list(models,"Outside PIs"))
for (i in 1:k){
  outsidePI[i] <- sum(lower[,i] > test | upper[,i] < test)
}
print(outsidePI)
