###################################################
# Principles of Business Forecasting 2e, 2017
# ord, K., Fildes, R. and Kourentzes, N.
# (Book website) - TO BE EDITED
#
# Companion scripts for R
###################################################

# Exercises 6.3

###################################################
# These line will be removed when this is packaged!
load("./data/data_SA_oil_prices_2.Rdata")
###################################################

# Store the data in a conveniently named variable
y <- data_SA_oil_prices_2
n <- length(y)

# Let us find how long is the evaluation set
test <- window(y,start=2008)
m <- length(test)

if (!require("forecast")){install.packages("forecast")}

# Initialise an array to store the forecasts
frc <- array(NA,c(m,1),dimnames=list(time(test),"ARIMA"))

# Produce forecasts
for (i in 1:m){
  # We run a rolling origin evaluation. At each iteration (i) we need to provide the current fitting sample
  # This must be a ts class object for the forecast package functions to work properly
  fitsample <- y[1:(n-m+i-1)]
  fitsample <- ts(fitsample,frequency=frequency(y),start=start(y))
  fit <- auto.arima(fitsample)
  # Produce the 1-step ahead forecast
  frc[i,1] <- forecast(fit,h=1)$mean
}

# To simplify plotting the results we convert frc into a ts object
frc <- ts(frc,frequency=frequency(test),start=start(test))

# Calculate error metrics
e <- test - frc
# Error metrics
RMSE <- sqrt(apply(e^2,2,mean))
RMdSE <- sqrt(apply(e^2,2,median))
MAE <- apply(abs(e),2,mean)
MdAE <- apply(abs(e),2,median)
MAPE <- apply(abs(e)/test,2,mean)*100
MdAPE <- apply(abs(e)/test,2,median)*100
E <- rbind(RMSE,MAE,MAPE,RMdSE,MdAE,MdAPE)
print(round(E,3))

