###################################################
# Principles of Business Forecasting 2e, 2017
# ord, K., Fildes, R. and Kourentzes, N.
# (Book website) - TO BE EDITED
#
# Companion scripts for R
###################################################

# Exercises 6.6

###################################################
# These line will be removed when this is packaged!
load("./data/data_US_Retail_Sales_2.Rdata")
###################################################

if (!require("forecast")){install.packages("forecast")}

# Store the data in a conveniently named variable
y <- data_US_Retail_Sales_2

z <- log(y)
# We recommend that you attempt a manual ARIMA model build using time series exploration, diagnostics and the ARIMA function
plot(z)
acf(z)
pacf(z)
# Strong evidence of seasonality
d12z <- diff(z,12)
plot(d12z)
acf(d12z)
pacf(d12z)
# A quicker way to get all these plots is the function tsdisplay from the forecast package
tsdisplay(d12z)
# Evidence of trend
d12d1z <- diff(d12z,1)
tsdisplay(d12d1z)
# There is potential overdifferencing! (Note the oscillations in ACF.) We should have tested if there was any need to difference.
# There are two useful functions in the forecast package: ndiffs and nsdiffs that test for the number of level and seasonal differences needed
# using various statistical tests. See ?ndiffs and ?nsdiffs
ndiffs(d12z)
# The test recommendation is to avoid additional differencing, which was the same impression we got from visual inspection.

# Now we fit some ARIMA models and to identify the best
fit <- vector("list",6)
# Note that we build the model on the original time series and instruct it to use differencing
fit[[1]] <- Arima(z,order=c(0,0,0),seasonal=c(0,1,0))
fit[[2]] <- Arima(z,order=c(1,0,0),seasonal=c(0,1,0))
fit[[3]] <- Arima(z,order=c(1,0,1),seasonal=c(0,1,0))
fit[[4]] <- Arima(z,order=c(0,0,0),seasonal=c(1,1,0))
fit[[5]] <- Arima(z,order=c(1,0,0),seasonal=c(1,1,0))
fit[[6]] <- Arima(z,order=c(1,0,1),seasonal=c(1,1,0))
# extract AIC from the fitted models
AIC <- unlist(lapply(fit,function(x){x$aic}))
print(AIC)
# Fit 6 is promising. Instead of modelling blind we can extract the residuals and explore them
resid6 <- residuals(fit[[6]])
tsdisplay(resid6)
# There are indications of remaining AR and MA terms. We try 3 additional modelling options:
fit[[7]] <- Arima(z,order=c(2,0,1),seasonal=c(1,1,0))
fit[[8]] <- Arima(z,order=c(1,0,2),seasonal=c(1,1,0))
fit[[9]] <- Arima(z,order=c(2,0,2),seasonal=c(1,1,0))
AIC <- unlist(lapply(fit,function(x){x$aic}))
print(AIC)
# Fit 9 seems to be the most promising. Let us check!
tsdiag(fit[[9]])
resid9 <- residuals(fit[[9]])
qqnorm(resid9)
qqline(resid9)
# We should explore additional models, before we accept ARIMA(2,0,2)(1,1,0) as the final one.

# At this point it is useful to check the suggestion of auto.arima, which broadly follows a similar search strategy
fit.auto <- auto.arima(z)
print(fit.auto)
# It suggestes an additional seasonal AR term
tsdiag(fit.auto)
resid <- residuals(fit.auto)
qqnorm(resid)
qqline(resid)
