###################################################
# Principles of Business Forecasting 2e, 2017
# ord, K., Fildes, R. and Kourentzes, N.
# (Book website) - TO BE EDITED
#
# Companion scripts for R
###################################################

# Exercise 7.4
# Exercise 7.5 is solved in the same way

###################################################
# These line will be removed when this is packaged!
load("./data/data_Netflix_2.Rdata")
###################################################

# Store the data in a conveniently named variable
y <- data_Netflix_2

y_estim <- as.vector(window(data_Netflix_2,start=c(2000,1),end=c(2012,4))[,1])
n <- length(y_estim)
x <- data.frame("time"=1:n,"growth"=y_estim)

# Preliminary analysis
cor(x)
plot(x)

# Regression sales on price
fit1 <- lm(growth ~ time, data=x)
summary(fit1)

# Get p-values and evaluate with a = 0.05
pval1 <- summary(fit1)$coefficients[,4]
pval1 <= 0.05

# Some useful plots are provided by plot(fit1)
plot(fit1)
# We can see that the fit is not great. It is also useful to plot actuals vs fitted
plot(x$growth,predict(fit1,x),xlab="Actuals",ylab="Predicted")
# If the fit was capturing the underlying trend than the plotted points should be on the diagonal line.

# Predict 2013:Q1 to 2015:Q4 and construct 95% PIs
xnew <- data.frame("time"=(n+1):(n+12))
frc <- predict(fit1,xnew,interval="prediction",level=0.95)

# Plot predictions
if (!require("RColorBrewer")){install.packages("RColorBrewer")}
cmp <- brewer.pal(3,"Set1")

test <- window(y[,1],start=c(2013,1),end=c(2015,4))
frc <- ts(frc,frequency=frequency(test),start=start(test))
yy <- range(c(test,frc))
plot(test,ylim=yy,ylab="Growth")
lines(frc[,1],col=cmp[1])
lines(frc[,2],col=cmp[2],lty=2)
lines(frc[,3],col=cmp[2],lty=2)
legend("topleft",c("Actuals","Forecast","95% PIs"),col=c("black",cmp[1:2]),lty=c(1,1,2,2))
