###################################################
# Principles of Business Forecasting 2e, 2017
# ord, K., Fildes, R. and Kourentzes, N.
# (Book website) - TO BE EDITED
#
# Companion scripts for R
###################################################

# Exercise 8.5

###################################################
# These line will be removed when this is packaged!
load("./data/data_Gas_prices_1.Rdata")
###################################################

# Store the data in a conveniently named variable
x <- data_Gas_prices_1

z <- x
z[,1:4] <- z[,1:4]/matrix(rep(z[,5],4),ncol=4)
# COnstruct lags for z
k <- dim(z)[2]
n <- dim(z)[1]
z <- cbind(z,rbind(rep(NA,k),z[-n,]))
colnames(z) <- c(colnames(x),paste0("L1_",colnames(x)))
z_estim <- window(z,end=c(2008,12))
z_test <- window(z,start=c(2009,1))
# COnstruct lags for x
x <- cbind(x,rbind(rep(NA,k),x[-n,]))
colnames(x) <- colnames(z)
x_estim <- window(x,end=c(2008,12))
x_test <- window(x,start=c(2009,1))

# Some initial exploration
plot(z_estim[,c(1,k+c(2:4,8))])
cor(z_estim[-1,c(1,k+c(2:4,8))])
plot(as.data.frame(z_estim[-1,c(1,k+c(2:4,8))]))

# Estimate regression with transformed variables
fit1 <- lm(unleaded~L1_crude_price+L1_sp500+L1_pdi+L1_unemp,data=z_estim)
summary(fit1)
# Validation
par(mfrow=c(2,2))
plot(fit1)
par(mfrow=c(1,1))

# Estimate regression with original data
fit2 <- lm(unleaded~L1_crude_price+L1_sp500+L1_pdi+L1_unemp,data=x_estim)
summary(fit2)

# To compare the two models we need to produce forecasts, but evaluate performance on the same data
# First we calculate the forecasts
frc1 <- predict(fit1,z_test)
frc2 <- predict(fit2,x_test)
# Transform frc1
frc1 <- frc1 * as.data.frame(z_test)$cpi

# CAlculate MAE
MAE <- apply(abs(matrix(rep(x_test[,1],2),ncol=2) - cbind(frc1,frc2)),2,mean)
names(MAE) <- c("w/ Tranformation","w/o Transformation")
print(MAE)

if (!require("RColorBrewer")){install.packages("RColorBrewer")}
cmp <- brewer.pal(3,"Set1")

yy <- range(c(x_test[,1],frc1,frc2))
plot(as.vector(x_test[,1]),ylim=yy,type="l")
lines(frc1,col=cmp[1])
lines(frc2,col=cmp[2])
legend("topleft",c("Actual",names(MAE)),col=c("black",cmp[1:2]),lty=1)
