###################################################
# Principles of Business Forecasting 2e, 2017
# ord, K., Fildes, R. and Kourentzes, N.
# (Book website) - TO BE EDITED
#
# Companion scripts for R
###################################################

# Exercise 8.9

###################################################
# These line will be removed when this is packaged!
load("./data/data_Gas_prices_1.Rdata")
###################################################

# Store the data in a conveniently named variable
x <- data_Gas_prices_1
k <- dim(x)[2]
n <- dim(x)[1]
lbls <- colnames(x)
# COnstruct lags for x
x <- cbind(x,rbind(rep(NA,k),x[-n,]))
colnames(x) <- c(lbls,paste0("L1_",lbls))
x_estim <- window(x,end=c(2008,12))
x_test <- window(x,start=c(2009,1))

fit <- lm(unleaded~L1_crude_price+L1_sp500+L1_rr_sales+L1_unemp,data=x_estim)
summary(fit)

# TO generate three step ahead forecasts we need to produce then iteratively so that the lag1 values become available
# For example, to generate the forecast for February 2009 we need the L1 values for January. These are actually available,
# as our estimation sample runs till December 2008, so with L1 it becomes January 2009. However to produce a forecast for
# March 2009 we need the February 2009 values, which are not avaiable and must be forecasted.

# There are many ways to produce these forecasts. We will rely on exponential smoothing.
if (!require("forecast")){install.packages("forecast")}
m <- dim(x_test)[1]

frc <- array(NA,c(m-3+1,3),dimnames=list(NULL,c("Forecast","Upper","Lower")))
for (i in 1:(m-3+1)){  # That is test set size - horizon + 1

  # z has the non-lagged values of crude_price and rr_sales up to the forecast origin
  z <- x[1:(n-m+i-1),c(2,6)]
  z <- ts(z,frequency=12,start=c(1996,1))
  f <- array(NA,c(2,2))
  for (j in 1:2){
    f[,j] <- forecast(ets(z[,j]),h=2)$mean
  }
  # The x's for the forecasted variables (lagged)
  xnew <- rbind(z[n-m+i-1,],f)
  colnames(xnew) <- paste0("L1_",colnames(xnew))
  # and the x's for the knwon variables
  xnew <- cbind(xnew,x[(n-m+i):(n-m+i+2),c(12,17)])
  # Produce t+3 forecasts and 90% prediction intervals
  xnew <- as.data.frame(xnew)
  frc[i,] <- predict(fit,xnew,interval="prediction",level=0.9)[3,]

}

# Visualise
if (!require("RColorBrewer")){install.packages("RColorBrewer")}
cmp <- brewer.pal(3,"Set1")

yy <- range(c(x_test[,1],frc))
plot(x_test[c(-1,-2),1],ylim=yy,type="l")
lines(frc[,1],col=cmp[1])
lines(frc[,2],col=cmp[2],lty=2)
lines(frc[,3],col=cmp[2],lty=2)

legend("topleft",c("Actual","Forecast t+3","Prediction Intervals"),col=c("black",cmp[1:2]),lty=c(1,1,2))
