###################################################
# Principles of Business Forecasting 2e, 2017
# ord, K., Fildes, R. and Kourentzes, N.
# (Book website) - TO BE EDITED
#
# Companion scripts for R
###################################################

# Exercise 9.12

###################################################
# These line will be removed when this is packaged!
load("./data/data_Gas_prices_2.Rdata")
###################################################

# Rename variable for convenience
x <- data_Gas_prices_2
# COnstruct lags - we need to increase these to t-3
k <- dim(x)[2]
n <- dim(x)[1]
lbls <- colnames(x)
# COnstruct lags for x
x <- cbind(x,rbind(rep(NA,k),x[-n,]))
colnames(x) <- c(lbls,paste0("L1_",lbls))
# Seperate samples
x_estim <- window(x,end=c(2008,12))
x_test <- window(x,start=c(2009,1),end=c(2010,12))

# There are two modelling approaches for this one
# First, fit a regression, extract the residuals and fit an ARIMA on those (note that in this case the AR term is on the residuals!)
# This approach has the disadvantage that the regression coefficients do not update as the ARIMA on the residuals is specified,
# and therefore these will most probably end up being suboptimal.

# We start from the model from which we calculate the residuals
fit <- lm(unleaded ~ L1_unleaded+L1_crude_price+L1_sp500+L1_unemp+L1_rr_sales,data=x_estim)
summary(fit)
resid <- fit$residuals

# We can now analyse the lag structure of the residuals
acf(resid)
# There is clearly a pattern
# At this point we can manually model this as a regression problem, or fit an ARIMA
if (!require("forecast")){install.packages("forecast")}
resid <- ts(resid,frequency=frequency(resid),end=end(x_estim))
fit_arima1 <- auto.arima(resid)
print(fit_arima1)
# Or we can restrict it to AR terms only
fit_arima2 <- auto.arima(resid,max.q=0)
print(fit_arima2)
# Which indicates that two lags of the residuals are adequate

# The sum of the forecasts of fit and fit.arima2 is the final forecast of the regARIMA
# Let us produce out-of-sample forecasts and evaluate on forecast accuracy
m <- dim(x_test)[1]
e <- array(NA,c(m,3))
for (i in 1:m){
  if (i == 1){
    tsend <- tail(time(x_estim[,1]),1)
  } else {
    tsend <- time(x_test[,1])[i-1]
  }
  tsend <- as.vector(tsend)
  fitsample <- window(x,end=tsend)
  frcsample <- window(x,start=tsend+deltat(x),end=tsend+deltat(x)) # Test is the next month
  # Refit models and calculate errors
  fitr <- lm(unleaded ~ L1_unleaded+L1_crude_price+L1_sp500+L1_unemp+L1_rr_sales,data=fitsample)
  resid <- fitr$residuals
  resid <- ts(resid,frequency=frequency(fitsample),start=start(fitsample))
  fit_arima1r <- Arima(resid,model=fit_arima1)
  fit_arima2r <- Arima(resid,model=fit_arima2)
  # Forecasts of regression and ARIMA parts
  frc <- predict(fitr,frcsample)
  frc_resid1 <- forecast(fit_arima1r,1)$mean
  frc_resid2 <- forecast(fit_arima2r,1)$mean
  # Calculate errors
  e[i,1] <- as.vector(frcsample[,1]) - frc
  e[i,2] <- as.vector(frcsample[,1]) - (frc + frc_resid1)
  e[i,3] <- as.vector(frcsample[,1]) - (frc + frc_resid2)
}

MAE <- apply(abs(e),2,mean)
print(round(MAE,3))

# Second, we can fit an ARIMAX, which is an ArIMA on the explanatory variable with regressors.
# The advantage is that all coefficients are optimised simultaneously and therefore are optimal, but
# the specification of such models can be quite cumbersome. In R there is the function auto.arima from the forecast package

fit <- vector("list",3)
fit[[1]] <- auto.arima(x_estim[,1],xreg=x_estim[,14])
print(fit[[1]])
# The automatic specification suggests no error lags
tsdiag(fit[[1]])
# Observe the ACF of residuals - indeed there is no evidence of short term autocorrelation structure

# We can manually increase the MA order
fit[[1]] <- Arima(x_estim[,1],order=c(1,0,0),xreg=x_estim[,14]) # This is the same as auto.arima. We re-estimate to make the forecasting code later on standardised
fit[[2]] <- Arima(x_estim[,1],order=c(1,0,1),xreg=x_estim[,14])
print(fit[[2]])
# All information criteria increase

fit[[3]] <- Arima(x_estim[,1],order=c(1,0,2),xreg=x_estim[,14])
print(fit[[3]])
# All information criteria increase

# Let us produce out-of-sample forecasts and evaluate on forecast accuracy
m <- dim(x_test)[1]
e <- array(NA,c(m,3))
fitr <- fit
for (i in 1:m){
  if (i == 1){
    tsend <- tail(time(x_estim[,1]),1)
  } else {
    tsend <- time(x_test[,1])[i-1]
  }
  tsend <- as.vector(tsend)
  fitsample <- window(x,end=tsend)
  frcsample <- window(x,start=tsend+deltat(x),end=tsend+deltat(x)) # Test is the next month
  # Refit models and calculate errors
  for (j in 1:3){
    fitr[[j]] <- Arima(fitsample[,1],model=fit[[j]],xreg=fitsample[,14])
    frc <- forecast(fitr[[j]],h=1,xreg=frcsample[,14])$mean
    e[i,j] <- as.vector(frcsample[,1]) - as.vector(frc)
  }
}

# Calculate MAE
MAE <- apply(abs(e),2,mean)
print(round(MAE,3))

# Indeed the model with the lagged errors in the most accruate for t+1 forecasts when modelled with ARIMAX
