###################################################
# Principles of Business Forecasting 2e, 2017
# ord, K., Fildes, R. and Kourentzes, N.
# (Book website) - TO BE EDITED
#
# Companion scripts for R
###################################################

# Exercise 9.17

###################################################
# These line will be removed when this is packaged!
load("./data/data_Gas_prices_2.Rdata")
###################################################

# Rename variable for convenience
x <- data_Gas_prices_2

# COnstruct lags
k <- dim(x)[2]
n <- dim(x)[1]
lbls <- colnames(x)
# COnstruct lags for x
x <- cbind(x,rbind(rep(NA,k),x[-n,]))
colnames(x) <- c(lbls,paste0("L1_",lbls))
# Seperate samples
x_estim <- window(x,end=c(2013,12))
x_test <- window(x,start=c(2014,1),end=c(2015,12))

# Fit OLS model
fit <- lm(unleaded ~ L1_crude_price+L1_unleaded+L1_sp500+L1_pdi+L1_rr_sales+L1_cpi+L1_housing+L1_unemp+L1_demand,data=x_estim)
summary(fit)

# For LASSO regression we can use the glmnet package
if (!require("glmnet")){install.packages("glmnet")}

lambda <- c(0.5,1,1.5,2)
q <- length(lambda)

fit_lasso <- vector("list",q+1)
cf <- array(NA,c(length(lbls)+1,q+1),dimnames=list(c("Intercept",paste0("L1_",lbls)),c(paste0("Lambda(",lambda,")"),"Lambda(Opt)")))

# Fit the three LASSO regressions with the pre-specified Lambda
# All pre- and post-processing of variables and coefficients is done internally
for (i in 1:q){
  fit_lasso[[i]] <- glmnet(x=x[-1,13:24],y=x[-1,1],lambda=lambda[i])
  cf[,i] <- as.vector(coef(fit_lasso[[i]]))
}
# FInd optimal lambda with cross-validation
fit_lasso[[q+1]] <- cv.glmnet(x=x[-1,13:24],y=x[-1,1])
cf[,q+1] <- as.vector(coef(fit_lasso[[q+1]]))
# Asking for a plot of this model gives us the cross-validated errors
plot(fit_lasso[[q+1]])
# If we ask for a plot of a standard lasso fit then we get the trajectories of the coefficients as shirnkage increases
plot(glmnet(x=x[-1,13:24],y=x[-1,1]))

print(round(cf,4))

# Construct forecasts for the hold-out
frc <- array(NA,c(dim(x_test)[1],q+2),dimnames=list(NULL,c("OLS",colnames(cf))))
frc[,1] <- predict(fit,x_test)
for (i in 1:(q+1)){
  frc[,1+i] <- predict(fit_lasso[[i]],x_test[,13:24])
}

# Calculate errors
e <- matrix(rep(x_test[,1],q+2),ncol=q+2) - frc
MAE <- apply(abs(e),2,mean)
print(round(MAE,3))
