###################################################
# Principles of Business Forecasting 2e, 2017
# ord, K., Fildes, R. and Kourentzes, N.
# (Book website) - TO BE EDITED
#
# Companion scripts for R
###################################################

# Exercises 9.4

###################################################
# These line will be removed when this is packaged!
load("./data/data_Walmart_2.Rdata")
###################################################

# Store the data in a conveniently named variable
x <- data_Walmart_2
n <- dim(x)[1]
# Model 5 (page 287) has L1_GDP, L1_Sales, L4_Sales and seasonal dummies
# We will add to x the required lags and dummies
lbls <- colnames(x)
# Lags 1
x1 <- rbind(rep(NA,2),x[1:(n-1),])
# Lags 4
x4 <- rbind(array(NA,c(4,2)),
            x[1:(n-4),])
# Seasonal dummies
d <- diag(rep(1,4))
d <- matrix(rep(d,ceiling(n/4)),ncol=4)[1:n,1:3]
# Merge data
z <- cbind(x,x1,x4,d)
colnames(z) <- c(lbls,paste0("L1_",lbls),paste0("L4_",lbls),paste0("D",1:3))
# Split into samples
z_estim <- window(z,end=c(2013,4))
z_test <- window(z,start=c(2014,1),end=c(2015,4))

# Fit model
fit <- lm(sales~L1_gdp+L1_sales+L4_sales+D1+D2+D3,data=z_estim)
summary(fit)

# One-step ahead forecasts
frc <- predict(fit,z_test,interval="prediction",level=.95)

# Calculate errors
e <- z_test[,1]-frc[,1]
MAE <- mean(abs(e))
MAPE <- mean(abs(e)/z_test[,1])*100
print(round(rbind(MAE,MAPE),3))

# Visualise forecasts
if (!require("RColorBrewer")){install.packages("RColorBrewer")}
cmp <- brewer.pal(3,"Set1")

frc <- ts(frc,frequency=frequency(z_test),start=start(z_test))
yy <- range(c(z[,1],frc))
plot(x[,1],ylab="Sales",ylim=yy)
lines(frc[,1],col=cmp[1])
lines(frc[,2],col=cmp[2],lty=2)
lines(frc[,3],col=cmp[2],lty=2)
legend("bottomright",c("Sales","Forecast","Prediction Intervals"),col=c("black",cmp[1:2]),lty=c(1,1,2))
