###################################################
# Principles of Business Forecasting 2e, 2017
# ord, K., Fildes, R. and Kourentzes, N.
# (Book website) - TO BE EDITED
#
# Companion scripts for R
###################################################

# Exercises 9.6

###################################################
# These line will be removed when this is packaged!
load("./data/data_Walmart_2.Rdata")
###################################################

# Store the data in a conveniently named variable
x <- data_Walmart_2
n <- dim(x)[1]
# Model 5 (page 287) has L1_GDP, L1_Sales, L4_Sales and seasonal dummies
# We will add to x the required lags and dummies
lbls <- colnames(x)
# Lags 1
x1 <- rbind(rep(NA,2),x[1:(n-1),])
# Lags 4
x4 <- rbind(array(NA,c(4,2)),
            x[1:(n-4),])
# Seasonal dummies
d <- diag(rep(1,4))
d <- matrix(rep(d,ceiling(n/4)),ncol=4)[1:n,1:3]
# Merge data
z <- cbind(x,x1,x4,d)
colnames(z) <- c(lbls,paste0("L1_",lbls),paste0("L4_",lbls),paste0("D",1:3))

# Apply differences of logs transformation
z[,1:6] <- rbind(rep(NA,6),diff(log(z[,1:6]),1))

# Split into samples
z_estim <- window(z,end=c(2013,4))
z_test <- window(z,start=c(2014,1),end=c(2015,4))

# Fit model
fit <- lm(sales~L1_gdp+L1_sales+L4_sales+D1+D2+D3,data=z_estim)
summary(fit)
# Model diagnostics
par(mfrow=c(2,2))
plot(fit)
par(mfrow=c(1,1))

# We can use the existing model to evaluate whether the sales lags are needed or not
# We can do that using stepwise regression - note that the stepwise is done on AIC
fit_step <- step(fit)
summary(fit_step)
# But this stepwise search is not restricted to keep always the GDP and seasonal dummies
# and evaluate only the inclusion of sales lags. We can do that by setting the scope of the
# model. Using the argument scope we set the `lower' model to consider, i.e. all the variables
# that will be included. We could also set the upper model, excluding some variables from the
# modelling altogether.
fit_step2 <- step(fit,scope=list(lower=~L1_gdp+D1+D2+D3))
summary(fit_step2)

# One-step ahead forecasts
frc <- vector("list",3)
frc[[1]] <- predict(fit,z_test,interval="prediction",level=.95)
frc[[2]] <- predict(fit_step,z_test,interval="prediction",level=.95)
frc[[3]] <- predict(fit_step2,z_test,interval="prediction",level=.95)

# Calculate errors
act <- z_test[,1]
e <- array(NA,c(length(act),3))
for (i in 1:3){
  e[,i] <- act - frc[[i]][,1]
}
MAE <- apply(abs(e),2,mean)
# MAPE cannot be meaningfully calculated, unless we reverse the differencing, since there is
# no meaningful zero in the actuals.
MAPE <- apply(abs(e)/matrix(rep(act,length(frc)),ncol=length(frc)),2,mean)
print(round(rbind(MAE,MAPE),3))
