###################################################
# Principles of Business Forecasting 2e, 2017
# ord, K., Fildes, R. and Kourentzes, N.
# (Book website) - TO BE EDITED
#
# Companion scripts for R
###################################################

# Exercises 9.8

###################################################
# These line will be removed when this is packaged!
load("./data/data_Walmart_2.Rdata")
###################################################

# Store the data in a conveniently named variable
x <- data_Walmart_2
n <- dim(x)[1]
# Model 5 (page 287) has L1_GDP, L1_Sales, L4_Sales and seasonal dummies
# We will add to x the required lags and dummies
lbls <- colnames(x)
# Lags 1
x1 <- rbind(rep(NA,2),x[1:(n-1),])
# Lags 4
x4 <- rbind(array(NA,c(4,2)),
            x[1:(n-4),])
# Seasonal dummies
d <- diag(rep(1,4))
d <- matrix(rep(d,ceiling(n/4)),ncol=4)[1:n,1:3]
# Indicators for the Great Recession
# We will attempt two options. One is an indicator that starts from June 2008 and remains (modelling an overall change after that point)
# The second option is to add a dummy only for the duration of the recession, Q3 2008 - Q3 2009 (end data from NBER)
tt <- time(x)
d2 <- as.numeric(tt >= 5/12+2008)
d3 <- as.numeric(tt >= 5/12+2008 & tt <= 3/4+2009)
# Merge data
z <- cbind(x,x1,x4,d,d2,d3)
colnames(z) <- c(lbls,paste0("L1_",lbls),paste0("L4_",lbls),paste0("D",1:3),"GR1","GR2")
# Split into samples
z_estim <- window(z,end=c(2013,4))
z_test <- window(z,start=c(2014,1),end=c(2015,4))

# Fit model - no recession
fit <- vector("list",3)
fit[[1]] <- lm(sales~L1_gdp+L1_sales+L4_sales+D1+D2+D3,data=z_estim)
summary(fit[[1]])
# and the two alternative recession approaches
fit[[2]] <- lm(sales~L1_gdp+L1_sales+L4_sales+D1+D2+D3+GR1,data=z_estim)
summary(fit[[2]])
fit[[3]] <- lm(sales~L1_gdp+L1_sales+L4_sales+D1+D2+D3+GR2,data=z_estim)
summary(fit[[3]])

# One-step ahead forecasts
frc <- vector("list",3)
for (i in 1:3){
  frc[[i]] <- predict(fit[[i]],z_test,interval="prediction",level=.95)
}

# Calculate errors
act <- z_test[,1]
e <- array(NA,c(length(act),3))
for (i in 1:3){
  e[,i] <- act - frc[[i]][,1]
}
MAE <- apply(abs(e),2,mean)
MAPE <- apply(abs(e)/matrix(rep(act,length(frc)),ncol=length(frc)),2,mean)*100
E <- rbind(MAE,MAPE)
colnames(E) <- c("w/o GR","GR1","GR2")
print(round(E,3))

# We can use the solutions from exercises 9.5-9.7 to experiment with different estimation samples,
# data transformations and variable selection
